import { init, loadRemote } from '@module-federation/enhanced/runtime';
import { DefaultApiClient } from './schema/openapi/generated/apis/Api.client.esm.js';
import { createFrontendFeatureLoader } from '@backstage/frontend-plugin-api';

function dynamicFrontendFeaturesLoader(options) {
  return createFrontendFeatureLoader({
    async loader({ config }) {
      const dynamicPLuginsConfig = config.getOptionalConfig("dynamicPlugins");
      if (!dynamicPLuginsConfig) {
        return [];
      }
      function error(message, err) {
        console.error(
          `${message}: ${err instanceof Error ? err.toString() : JSON.stringify(err)}`
        );
      }
      const backendBaseUrl = config.getString("backend.baseUrl");
      const appPackageName = config.getOptionalString("app.packageName") ?? "app";
      let frontendPluginRemotes;
      try {
        const apiClient = new DefaultApiClient({
          discoveryApi: {
            getBaseUrl: async (rootPath) => `${backendBaseUrl}/${rootPath}`
          },
          fetchApi: {
            fetch(input) {
              return global.fetch(input);
            }
          }
        });
        const response = await apiClient.getRemotes({});
        if (!response.ok) {
          throw new Error(`${response.status} - ${response.statusText}`);
        }
        frontendPluginRemotes = await response.json();
      } catch (err) {
        error(
          `Failed fetching module federation configuration of dynamic frontend plugins`,
          err
        );
        return [];
      }
      try {
        init({
          ...options?.moduleFederation,
          name: appPackageName.replaceAll("@", "").replaceAll("/", "__").replaceAll("-", "_"),
          remotes: frontendPluginRemotes.map((remote) => ({
            alias: remote.packageName,
            ...remote.remoteInfo
          }))
        });
      } catch (err) {
        error(`Failed initializing module federation`, err);
        return [];
      }
      const features = (await Promise.all(
        frontendPluginRemotes.map(async (remote) => {
          console.debug(
            `Loading dynamic plugin '${remote.packageName}' from '${remote.remoteInfo.entry}'`
          );
          const moduleFeatures = await Promise.all(
            remote.exposedModules.map(async (exposedModuleName) => {
              const remoteModuleName = exposedModuleName === "." ? remote.remoteInfo.name : `${remote.remoteInfo.name}/${exposedModuleName}`;
              let module;
              try {
                module = await loadRemote(remoteModuleName);
              } catch (err) {
                error(
                  `Failed loading remote module '${remoteModuleName}' of dynamic plugin '${remote.packageName}'`,
                  err
                );
                return void 0;
              }
              if (!module) {
                console.warn(
                  `Skipping empty dynamic plugin remote module '${remoteModuleName}'.`
                );
                return void 0;
              }
              console.info(
                `Remote module '${remoteModuleName}' of dynamic plugin '${remote.packageName}' loaded from ${remote.remoteInfo.entry}`
              );
              const defaultEntry = module.default;
              if (!isLoadable(defaultEntry)) {
                console.debug(
                  `Skipping dynamic plugin remote module '${remote}' since it doesn't export a new 'FrontendFeature' as default export.`
                );
                return void 0;
              }
              return defaultEntry;
            })
          );
          return moduleFeatures;
        })
      )).flat().filter((feature) => feature !== void 0);
      return [...features];
    }
  });
}
function isLoadable(obj) {
  if (obj !== null && typeof obj === "object" && "$$type" in obj) {
    return obj.$$type === "@backstage/FrontendPlugin" || obj.$$type === "@backstage/FrontendModule";
  }
  return false;
}

export { dynamicFrontendFeaturesLoader };
//# sourceMappingURL=loader.esm.js.map
